;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; this program reads in data from the bhps wave a (1991) and projects it forward over time
; program modified from simple3 which was used to validate copying procedure in q5 report
; this version is to validate the new parallel copying procedure on the same data
; real wage inflation is read in for each year.
; last updated 16.3.11
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; set up scenario
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to set_scenario
  ;if years = 0 [set change_in_real_income% 0.07]
  ;if years = 1 [set change_in_real_income% -4.21]
  ;if years = 2 [set change_in_real_income% 2.59]  
  ;if years = 3 [set change_in_real_income% 2.34]  
  ;if years = 4 [set change_in_real_income% 2.82] 
  ;if years = 5 [set change_in_real_income% 2.95]  
  ;if years = 6 [set change_in_real_income% -0.44]  
  ;if years = 7 [set change_in_real_income% 1.03]  
  ;if years = 8 [set change_in_real_income% 3.75]   
  ;if years = 9 [set change_in_real_income% 3.59]
  ;if years = 10 [set change_in_real_income% 1.04]   
  ;if years = 11 [set change_in_real_income% 1.72]
  ;if years = 12[set change_in_real_income% -1.24]
  ;if years = 13 [set change_in_real_income% 0.36]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; variable seclerations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

globals [
  pop                  ; number of individuals in the simulation
  years                ; number of simulated years elapsed
  hsize                ; mean number of individuals in household
  houses               ; number of simulated households
  currentcase          ; current individual id
  currenthouse         ; current household id
  maxhouseholds        ; highest household id
  totfood              ; total spent on food for whole population
  tothousing           ; total spent on housing
  totfuel              ; total spent on fuel
  income               ; total income of all households
  pcgas                ; mean budget share for gas
  pcfood               ; mean budget share for food
  pchousing            ; mean budget share for housing
  pcfuel               ; mean budget share for fuel
  pcelectric           ; mean budget share for electricity
  totmortgage          ; total monthly mortgage payments for population
  meanmortgage         ; mean household monthly mortgage payments
  pcmortgage           ; budget share of income for mortgage
  pcsingle             ; proportion of population single
  pcmarried            ; proportion of population married
  pccohab              ; proportion of population cohabiting
  pcseparated          ; proportion of population separeted
  pcdivorced           ; proportion of population divorced
  pcwidowed            ; proportion of population widowed
  nmales               ; number of males simulated
  nfem                 ; number of females simulated
  h1                   ; proportion of households with 1 occupant
  h2                   ; proportion of households with 2 occupants
  h3                   ; proportion of households with 3 occupants
  h4                   ; proportion of households with 4 or more occupants
  totgas               ; total expenditure on gas
  totoil               ; total expenditure on oil
  totelectric          ; total expenditure on electricity
  meanfood             ; mean household expenditure on food
  meanelectric         ; mean household expenditure on electricity
  meangas              ; mean household expenditure on gas
  meanhousing          ; mean household expenditure on housing
  pclow                ; proportion of households with annual income less than 10000
  pcmedium             ; proportion of households with annual income 10000 to 20000
  pchigh               ; proportion of households with annual income over 20000
  low                  ; number of households with annual income less than 10000
  medium               ; number of households with annual income 10000 to 20000
  high                 ; number of households with income over 20000
  othergroup           ; number of type 6 households
  type1pc              ; proportion of type 1 households
  type2pc              ; proportion of type 2 households
  type3pc              ; proportion of type 3 households
  type4pc              ; proportion of type 4 households
  type5pc              ; proportion of type 5 households
  type6pc              ; proportion of type 5 households
  mean_population_income    ; current average income for the whole population
  target_mean_population_income  ; what the average population income will be after the income change has been effected
    
  gtempaxpmg    ; temporary variables for copying expenditures
  gtemparegion
  gtempahstype
  gtempahsval
  gtemparentg
  gtempaxpfood
  gtempafihhyr
  gtempelectricity
  gtempgas
  gtemphousing
  gtempoil
]

patches-own [
  pcase               ; patch address
  weight              ; household weight
  occupied            ; patch currently occupied 
  currenttype         ; current household classification number
  lasttype            ; previous classification
  oldhhincome         ; household income last year 
  numoccupants        ; number of people in the household
  males               ; number of males in household 
  females             ; number of females in household 
  food                ; spending on food
  fuel                ; spending on fuel
  young               ; number of under 30s
  midage              ; number aged 30 to 60
  old                 ; number over 60 
  housesize           ; number in house 
  axpmg               ; monthly mortgage
  aregion             ; region 
  ahstype             ; type of accommodation 
  ahsval              ; value of property
  arentg              ; gross rent
  axpfood             ; weekly food & grocery bill
  afihhyr             ; annual household income
  hhincome            ; household income sum of individual incomes within household
  electricity         ; weekly expenditure on electricity
  gas                 ; weekly expenditure on gas
  housing             ; net monthly housing costs
  oil                 ; monthly expenditure on oil
  targetincome        ; income after annual change  
  changes_pending     ; household expenditures should be updated this year
  
  ptempaxpmg          ; temporary variables for copying expenditures
  ptemparegion
  ptempahstype
  ptempahsval
  ptemparentg
  ptempaxpfood
  ptempafihhyr
  ptempelectricity
  ptempgas
  ptemphousing
  ptempoil  
]

turtles-own [
  case           ; case number
  partnerid      ; who number of partner
  parentid       ; who number of parent
  person         ; household member number
  homex          ; x coordinate of original address
  homey          ; y coordinate of original address
  age            ; years
  under20        ; aged under 20
  twenties       ; aged in twenties
  thirties       ; aged in thirties
  forties        ; aged in forties
  fifties        ; aged in fifties
  sixties        ; aged 60 and over  
  sex            ; male female
  nsex           ; numerical sex read from file
  nstatus        ; numerical marrital status read from file
  status         ; married cohabiting single widowed divorced separated samesex
  previousstatus ; previous marital status
  newwidow       ; widowed in current mortality module
  newlyseparated ; 0 = no: number indicates time since separation in years
  children       ; number of children born by agent
  numdependents  ; number of agents responsible for
  newbirth       ; birth in current fertility module
  numinhouse     ; number of individuals in the household
  student        ; currently a full time student
  education      ; age completed full time education
  changed        ; recent change indicator
  newcohabit     ; cohabitation began in current year
  nemployment    ; numerical employment status from file
  employment     ; type of employment
  ajbhrs         ; hours worked per week
  asaved         ; amount saved each month
  apaygyr        ; gross earnings last 12 months
  atenure        ; housing tenure
  ajbpl          ; work location
  ajbttwm        ; means of travel to work
  ajbttwt        ; minutes spent travelling to work
]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; initialisation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup
  type "loading data"
  clear-all
  init-patches            ; read household level data
  initialise              ; read individual level data
  applyhouseholdweights
  findparents
  set years 0
  set change_in_real_income% 0
  clear_patches
  occupy_patches
  initial_household_classification  ; set type of household based on demographics
  initialise_temp_expenditures      ;  avoid copying zero values when change_expenditures runs for the first time
  ask patches [if any? turtles-here [calculateexpenditures]]  ; set new household income
  summarise                         ; prepare outputs
  outputexpenditures
  update-plot
end

to init-patches        ; read in household data
  let numhouses 0      ; house counter
  let lasthouse 0      ; highest house so far
  set currenthouse 0   ; number of current household
  ask patches [set pcase 0]
  file-open "housefile.txt"
  while [not file-at-end?] [
    set currenthouse file-read
      if currenthouse > lasthouse [
      set numhouses numhouses + 1
      if numhouses > households  [file-close stop]
      ask one-of patches with [pcase = 0] [
        set pcase currenthouse
        set weight file-read
        set housesize file-read        ; number of people in household from file
        set axpmg file-read            ; monthly mortgage payment
        set aregion file-read          ; UK region
        set ahstype file-read          ; BHPS household type
        set ahsval file-read           ; value of household
        set arentg file-read           ; gross rent
        set axpfood file-read          ; weekly food & grocery bill
        set afihhyr file-read          ; annual household income
        set electricity file-read      ; weekly expenditure on electricity
        set gas file-read              ; weekly expenditure on gas
        set housing file-read          ; monthly expenditure on housing
        set oil file-read              ; monthly expenditure on oil

        if afihhyr <= 0 [set afihhyr 0]
        if housesize < 0 [set housesize 0]        
        if axpmg < 0 [set axpmg 0]
        if arentg < 0 [set arentg 0]                
        if axpfood < 0 [set axpfood 0]
        if electricity < 0 [set electricity 0]        
        if gas < 0 [set gas 0]
        if housing < 0 [set housing 0]        
        if oil < 0 [set oil 0]                        
      ]
      set lasthouse currenthouse
    ]  
  ]
  file-close
  write "finished reading household file"
end

to initialise  ; set up initial agent population
  let r 0.0
  let recnum 0
  let finished 0
  set maxhouseholds max [pcase] of patches
  file-open "indfile.txt"
  while [not file-at-end? and finished = 0] [
    set recnum file-read
    if-else recnum <= maxhouseholds [
      cro 1 [                  ; create a new agent
        set case recnum        ; set turtle identifier from file
        set currentcase case   ; set current case number
        set person file-read         ; individual id
        set nsex file-read           ; gender
        set nstatus file-read        ; marital status
        set age file-read            ; age in years
        set children file-read       ; number of dependents
        set nemployment file-read    ; employment status
        set education file-read      ; age completed full time education
        set ajbhrs file-read         ; hours worked per week
        set asaved file-read         ; amount saved pe month
        set apaygyr file-read        ; gross earnings in the last year
        set atenure file-read        ; housing tenure
        set ajbpl file-read          ; work location
        set ajbttwm file-read        ; means of travelling to work
        set ajbttwt file-read        ; minutes spent travelling to work

        setagebands  ; initialise or update age bands according to current age
        
        if nsex = 1 [set sex "male" set color blue]
        if nsex = 2 [set sex "female" set color pink]
      
        if nstatus = 0 [set status "single"]
        if nstatus = 1 [set status "married"]
        if nstatus = 2 [set status "cohabiting"]
        if nstatus = 3 [set status "widowed"]
        if nstatus = 4 [set status "divorced"]
        if nstatus = 5 [set status "separated"]
        if nstatus = 6 [set status "single"]
        if nstatus = 7 [set status "same sex couple"]
        if nstatus < 0 [set status "single"]
        
        set previousstatus "single"
        if status = "separated" [set previousstatus "married"]
        if status = "widowed" [set previousstatus "married"]
        if status = "divorced" [set previousstatus "separated"]
        
        set student 0
        if age <= 16 [set student 1]
        
        if nemployment <= 0 [set employment "Unknown"]
        if nemployment = 1 [set employment "Self Employed"]
        if nemployment = 2 [set employment "Employed"]
        if nemployment = 3 [set employment "Unemployed"]
        if nemployment = 4 [set employment "Retired"]
        if nemployment = 5 [set employment "Family Care"]
        if nemployment = 6 [set employment "Full Time Student" set student 1]
        if nemployment = 7 [set employment "Long Term Sick / Disabled"]
        if nemployment >= 8 [ set employment "Other"]
        
        ifelse any? patches with [pcase = currentcase] [
          move-to one-of patches with [pcase = currentcase]  ; go to the family address
          set homex pxcor          ; initialise original address
          set homey pycor          ; initialise original address
        ]
        [show "patch not found" die]
      ]
    ]    
    [set finished 1]
  ]
  file-close
  show "finished reading agent file"
end

to clear_patches
  ask patches with [count turtles-here = 0] [clear_patch]
end

to occupy_patches
  ask patches with [count turtles-here > 0] [set occupied 1]
end

to clear_patch          ; reset patch attributes
  set occupied 0
  set pcase 0
  set weight 0
  set currenttype 0
  set lasttype 0
  set oldhhincome 0
  set numoccupants 0
  set males 0
  set females 0
  set food 0
  set housing 0
  set fuel 0
  set young 0
  set midage 0
  set old 0
  set housesize 0
  set axpmg 0
  set aregion 0
  set ahstype 0
  set ahsval 0
  set arentg 0
  set axpfood 0
  set afihhyr 0
  set electricity 0
  set gas 0
  set housing 0
  set oil 0
end

to applyhouseholdweights                        ; weight population to correspond to uk
  let r 0.0
  ask patches with [count turtles-here > 0] [
    set r random-float 1
    if weight = 0 [delete_household]        ; ignore households with zero weight
    if weight < 1 [if r > weight [delete_household]]  ; select households according to probability derived from weight
    if weight > 1 and weight < 2 [if r < (weight - 1) [store_expenditures copy_household copy_turtles]]
    if weight >= 2 [
      store_expenditures
      copy_household              ; duplicate household as many times as requires according to weight
      copy_turtles
      if r < (weight - 2) [
        store_expenditures
        copy_household
        copy_turtles
      ]
    ]    
  ]
  show "weights applied"
end

to delete_household
  ask turtles-here [die]
  clear_patch
end

to copy_household
  set currenthouse currenthouse + 1
  ask one-of patches with [pcase = 0] [
    read_expenditures
    set pcase currenthouse
  ]
end

to copy_turtles
  ask turtles-here [
    hatch 1 [
      set color white
      move-to one-of patches with [pcase = currenthouse]
      set homex pxcor          ; initialise original address
      set homey pycor          ; initialise original address      
    ]
  ]
end

to findparents          ; record id of parent
  let tempid -1
  let tempage 0
  let r 0.0
  set r random-float 1
  
  if r < 0.9 [                                              ; female is responsible for children
    ask patches with [count turtles-here > 0] [             ; for each household
      ask turtles-here [                                    ; for each occupant
        set tempid -1
        set tempage age
        if any? turtles-here with [sex = "female" and age > tempage + 16 and age < tempage + 40]        ; identify female parent
        [
          ask one-of turtles-here with [sex = "female" and age > tempage + 16 and age < tempage + 40] [set tempid who]  ; get parents id
        ]  
        set parentid tempid  ; remember parents id
      ]
    ]
  ]
  
  if r >= 0.9 [                                            ; male is responsible for children
    ask patches with [count turtles-here > 0] [            ; for each household
      ask turtles-here [                                   ; for each occupant
        set tempid -1
        set tempage age
        if any? turtles-here with [sex = "male" and age > tempage + 16 and age < tempage + 40]        ; identify male parent
        [
        ask one-of turtles-here with [sex = "male" and age > tempage + 16 and age < tempage + 40] [set tempid who]  ; get parents id
        ]  
        set parentid tempid  ; remember parents id
      ]    
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;control routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to go                  ; main loop
  if years = stop_after [stop]
  set_scenario
  clear_patches
  occupy_patches
  set mean_population_income mean [afihhyr] of patches with [occupied = 1]
  set target_mean_population_income mean_population_income + mean_population_income * (change_in_real_income% / 100)      
show change_in_real_income%
  update_expenditures          ; if household has changed income, copy expenditure pattern from household with similar new income 
  ask patches [
    if any? turtles-here [
      calculateexpenditures    ; update household income according to inflation rate
    ]
  ]
  update_expenditures          ; if state transition copy expenditures from another patch
  change_expenditures          ; copy expenditures from temporary patch variables after annual processing is complete
  summarise                    ; prepare outputs
  outputexpenditures
  update-plot
  set years years + 1          ; increment calender
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; expenditure modules
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to initial_household_classification                ; assign initial household type based on demographic parameters only
  ask patches with [count turtles-here > 0] [
    set lasttype currenttype
    set currenttype 6  ; other households
    
    if count turtles-here = 1 and count turtles-here with [sex = "male"] = 1 and
      max[age] of turtles-here < 65 [set currenttype 1]                                      ; single non pensionable
    if count turtles-here = 1 and count turtles-here with [sex = "female"] = 1 and
      max[age] of turtles-here < 60 [set currenttype 1]                                      ; single non pensionable

    if count turtles-here = 1 and count turtles-here with [sex = "male"] = 1 and
      min[age] of turtles-here >= 65 [set currenttype 2]                                     ; single pensionable
    if count turtles-here = 1 and count turtles-here with [sex = "female"] = 1 and
       min[age] of turtles-here >= 60 [set currenttype 2]                                    ; single pensionable
      
    if count turtles-here = 2 and count turtles-here with [age < 16] = 0 [set currenttype 3] ; 2 adults
    
    let nk 0
    set nk count turtles-here with [age < 16] ; count under 16s
    if nk >= 1 and count turtles-here = nk + 2 [set currenttype 4]                           ; 2 adults + children
    
    set nk count turtles-here with [age < 16] ; count under 16s
    if nk >= 1 and count turtles-here = nk + 1 [set currenttype 5]                           ; 1 adult + children
  ]    
end

to update_expenditures    ; if state transition copy expenditures from another patch
  let t 0                 ; household type
  let targ 0.0            ; household income after change
  let r 0.0               ; random number
  let copytype 0          ; type of household to copy from
  let keepgoing false     ; more households to process
  
  ask patches with [occupied = 1] [
    set keepgoing false
    set mean_population_income mean [ptempafihhyr] of patches with [occupied = 1]

    ; keep copying households until the population income reaches the level
    ; it should be when the intended income change has been applied
    
    if change_in_real_income% > 0 [  ; increasing incomes
      if mean_population_income < target_mean_population_income [set keepgoing true]
    ]  
    if change_in_real_income% < 0 [  ; decreasing incomes
      if mean_population_income > target_mean_population_income [set keepgoing true]  
    ]
    
    if afihhyr != targetincome and keepgoing = true [  ; if household income has changes
      set t currenttype
      set targ targetincome
      set r random-float 1
      set copytype 0
      
      if r < 0.5 [
        if any? other patches with [currenttype = t and afihhyr >= targ and occupied = 1] [set copytype 1]
  
        if copytype = 1 [  ; copy from next higher income household
          ask min-one-of other patches with [currenttype = t and afihhyr >= targ and occupied = 1] [afihhyr] [store_expenditures]
          read_temp_expenditures
        ]                          
        if copytype = 0 [
          vary_expenditures    ; if there is no suitable household to copy from then change income end expenditures in proportion
        ]
      ]
      if r >= 0.5 [
        if any? other patches with [currenttype = t and afihhyr <= targ and occupied = 1] [set copytype 1]
 
        if copytype = 1 [     ; copy from next lower income household
          ask max-one-of other patches with [currenttype = t and afihhyr <= targ and occupied = 1] [afihhyr] [store_expenditures]
          read_temp_expenditures
        ]          
        if copytype = 0 [
          vary_expenditures    ; if there is no suitable household to copy from then change income end expenditures in proportion
        ]
      ]
    ]
  ]
end

to vary_expenditures  ; inflate or deflate all expenditures when there is no higher or lower income household to copy from
  if afihhyr != 0 [
    set changes_pending true
    set ptempafihhyr afihhyr - (afihhyr * ((afihhyr - targetincome) / afihhyr))
    set ptempaxpmg axpmg - (axpmg * ((afihhyr - targetincome) / afihhyr)) 
    set ptempahsval ahsval - (ahsval * ((afihhyr - targetincome) / afihhyr))
    set ptemparentg arentg - (arentg * ((afihhyr - targetincome) / afihhyr))
    set ptempaxpfood axpfood - (axpfood * ((afihhyr - targetincome) / afihhyr))
    set ptempelectricity electricity - (electricity * ((afihhyr - targetincome) / afihhyr))
    set ptempgas gas - (gas * ((afihhyr - targetincome) / afihhyr))
    set ptemphousing housing - (housing * ((afihhyr - targetincome) / afihhyr))
    set ptempoil oil - (oil * ((afihhyr - targetincome) / afihhyr))
  ]  
end

to store_expenditures    ; record expenditures in global variables for copying to another patch
  set gtempaxpmg axpmg
  set gtempahsval ahsval
  set gtemparentg arentg
  set gtempaxpfood axpfood
  set gtempafihhyr afihhyr
  set gtempelectricity electricity
  set gtempgas gas
  set gtemphousing housing
  set gtempoil oil
end

to read_temp_expenditures      ; read stored global budget shares into current patch temporary variables
  set changes_pending true
  set ptempaxpmg gtempaxpmg
  set ptempahsval gtempahsval
  set ptemparentg gtemparentg
  set ptempaxpfood gtempaxpfood
  set ptempafihhyr gtempafihhyr
  set ptempelectricity gtempelectricity
  set ptempgas gtempgas
  set ptemphousing gtemphousing
  set ptempoil gtempoil
end

to read_expenditures    ; alter expenditures directly from global variables
  set axpmg gtempaxpmg
  set ahsval gtempahsval
  set arentg gtemparentg
  set axpfood gtempaxpfood
  set afihhyr gtempafihhyr
  set electricity gtempelectricity
  set gas gtempgas
  set housing gtemphousing
  set oil gtempoil
end

to change_expenditures    ; alter expenditures from temporary patch variables after annual processing is completed
  ask patches with [occupied = 1] [ ;and changes_pending = true] [
    set axpmg ptempaxpmg
    set ahsval ptempahsval
    set arentg ptemparentg
    set axpfood ptempaxpfood
    set afihhyr ptempafihhyr
    set electricity ptempelectricity
    set gas ptempgas
    set housing ptemphousing
    set oil ptempoil 
  ]
end

to initialise_temp_expenditures    ; set up initial values in temporary patch variables
  ask patches with [occupied = 1] [
    set ptempaxpmg axpmg
    set ptempahsval ahsval
    set ptemparentg arentg
    set ptempaxpfood axpfood
    set ptempafihhyr afihhyr
    set ptempelectricity electricity
    set ptempgas gas
    set ptemphousing housing
    set ptempoil oil
  ]  
end

to summarise                    ; set up global variables after annual cycle
  set pop count turtles
  if pop = 0 [stop]
  set houses count patches with [count turtles-here > 0]
  set h1 count patches with [count turtles-here = 1]
  set h1 (h1 / houses) * 100
  set h2 count patches with [count turtles-here = 2]
  set h2 (h2 / houses) * 100
  set h3 count patches with [count turtles-here = 3]
  set h3 (h3 / houses) * 100
  set h4 count patches with [count turtles-here >= 4]
  set h4 (h4 / houses) * 100
  set hsize (pop / houses)
  set nmales count turtles with [sex = "male"]
  set nfem count turtles with [sex = "female"]
  set pcsingle count turtles with [status = "single"]
  set pcsingle (pcsingle / pop) * 100
  set pccohab count turtles with [status = "cohabiting"]
  set pccohab (pccohab / pop) * 100
  set pcmarried count turtles with [status = "married"]
  set pcmarried (pcmarried / pop) * 100
  set pcseparated count turtles with [status = "separated"]
  set pcseparated (pcseparated / pop) * 100
  set pcdivorced count turtles with [status = "divorced"]
  set pcdivorced (pcdivorced / pop) * 100  
  set pcwidowed count turtles with [status = "widowed"]
  set pcwidowed (pcwidowed / pop) * 100  
end

to calculateexpenditures                                ; done for each patch
  set males count turtles-here with [sex = "male"]
  set females count turtles-here with [sex = "female"]
  set young count turtles-here with [age < 30]
  set midage count turtles-here with [age <= 30 and age < 60]
  set old count turtles-here with [age >= 60]
  set oldhhincome afihhyr
  set hhincome sum [apaygyr] of turtles-here
  set changes_pending false
  set targetincome afihhyr + afihhyr * (change_in_real_income% / 100)  ; set new household income
  if oldhhincome < 0 [show "error"]
end

to outputexpenditures                                  ; set up expenditures ready for plotting
  set houses count patches with [count turtles-here > 0]
  set totfood sum [axpfood] of patches with [occupied = 1]
  set totfood totfood * 4        ; multiply weekly food to make monthly
  set tothousing sum [housing] of patches with [occupied = 1]
  set totmortgage sum [axpmg] of patches with [occupied = 1]
  set income sum[afihhyr] of patches with [occupied = 1]
  set income income / 12              ; total monthly household income
  set totgas sum[gas] of patches with [occupied = 1]
  set totoil sum[oil] of patches with [occupied = 1]
  set totoil totoil / 4
  set totelectric sum[electricity] of patches with [occupied = 1]
  set meanfood (totfood / houses)        ; multiply to make monthly
  set meanelectric (totelectric / houses)
  set meangas (totgas / houses)
  set meanhousing tothousing / houses
  set meanmortgage (totmortgage / houses)
  set pcgas (totgas / income) * 100
  set pchousing (tothousing / income) * 100
  set pcfood (totfood / income) * 100
  set pcelectric (totelectric / income) * 100
  set pcmortgage (totmortgage / income) * 100

  set low count patches with [afihhyr >= 0 and afihhyr < 10000 and occupied = 1]
  set medium count patches with [afihhyr >= 10000 and afihhyr < 20000 and occupied = 1]
  set high count patches with [afihhyr > 20000 and occupied = 1]
  set othergroup count patches with [currenttype = 6 and occupied = 1]

  set pclow (low / houses) * 100
  set pcmedium (medium / houses) * 100
  set pchigh (high / houses) * 100

  set type1pc ((count patches with [currenttype = 1 and occupied = 1]) / houses) * 100
  set type2pc ((count patches with [currenttype = 2 and occupied = 1]) / houses) * 100
  set type3pc ((count patches with [currenttype = 3 and occupied = 1]) / houses) * 100 
  set type4pc ((count patches with [currenttype = 4 and occupied = 1]) / houses) * 100
  set type5pc ((count patches with [currenttype = 5 and occupied = 1]) / houses) * 100 
  set type6pc ((count patches with [currenttype = 6 and occupied = 1]) / houses) * 100
end

to followparent
  let tempid 0
  let r 0.0
  set r random-float 1
  if r < 0.5 [
    ask turtles with [age <= 25 and status != "married" and status != "cohabiting" and student = 0 and parentid > 0] [
      set tempid parentid
      if any? turtles with [who = tempid] [move-to turtle parentid]
    ]  
  ]  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; look up tables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setagebands
  set under20 0 set twenties 0 set thirties 0
  set forties 0 set fifties 0 set sixties 0
        
  if age < 20 [set under20 1]
  if age >= 20 and age < 30 [set twenties 1]
  if age >= 30 and age < 40 [set thirties 1]
  if age >= 40 and age < 50 [set forties 1]
  if age >= 50 and age < 60 [set fifties 1]
  if age >= 60 [set sixties 1]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; plotting section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to update-plot
  set-current-plot "population"
  plot pop
  
  set-current-plot "housesize"
  plot hsize
  
  set-current-plot "Mean Monthly Household Income"
  plot (income / houses)
  
  set-current-plot "Share of Income %"
  set-current-plot-pen "pcfood"
  plot pcfood
  set-current-plot-pen "pchousing"
  plot pchousing
  set-current-plot-pen "pcgas"
  plot pcgas
  set-current-plot-pen "pcelectricity"
  plot pcelectric
  set-current-plot-pen "pcmortgage"
  plot pcmortgage
  
  set-current-plot "Population Makeup"
  set-current-plot-pen "pcsingle"
  plot pcsingle
  set-current-plot-pen "pcmarried"
  plot pcmarried
  set-current-plot-pen "pccohab"
  plot pccohab
  set-current-plot-pen "pcseparated"
  plot pcseparated
  set-current-plot-pen "pcdivorced"
  plot pcdivorced  
  set-current-plot-pen "pcwidowed"
  plot pcwidowed
  
  set-current-plot "Household Size"
  set-current-plot-pen "h1"
  plot h1
  set-current-plot-pen "h2"
  plot h2
  set-current-plot-pen "h3"
  plot h3
  set-current-plot-pen "h4"
  plot h4
  
  set-current-plot "incomeband"
  set-current-plot-pen "pclow"
  plot pclow
  set-current-plot-pen "pcmedium"
  plot pcmedium
  set-current-plot-pen "pchigh"
  plot pchigh
end
@#$#@#$#@
GRAPHICS-WINDOW
293
365
1313
2406
50
100
10.0
1
10
1
1
1
0
0
0
1
-50
50
-100
100
0
0
1
ticks

CC-WINDOW
5
2420
1322
2515
Command Center
0

BUTTON
23
10
87
66
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
100
10
163
66
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

PLOT
19
755
278
891
Population
Years
Individuals
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16777216 true

PLOT
22
899
279
1034
Housesize
Years
Individuals
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16777216 true

PLOT
20
93
314
243
Share of Income %
Years
%
0.0
10.0
0.0
10.0
true
true
PENS
"pcfood" 1.0 0 -13840069 true
"pchousing" 1.0 0 -2674135 true
"pcgas" 1.0 0 -1184463 true
"pcelectricity" 1.0 0 -13345367 true
"pcmortgage" 1.0 0 -8630108 true

INPUTBOX
181
10
272
70
Households
100
1
0
Number

INPUTBOX
293
10
361
70
stop_after
50
1
0
Number

MONITOR
21
254
92
299
NIL
meanfood
0
1
11

MONITOR
21
310
91
355
NIL
meangas
0
1
11

MONITOR
106
255
180
300
NIL
meanhousing
0
1
11

MONITOR
209
256
266
301
NIL
years
0
1
11

PLOT
319
93
575
243
Population Makeup
Years
%
0.0
10.0
0.0
10.0
true
true
PENS
"pcmarried" 1.0 0 -16777216 true
"pcsingle" 1.0 0 -13791810 true
"pccohab" 1.0 0 -1184463 true
"pcseparated" 1.0 0 -10899396 true
"pcwidowed" 1.0 0 -7500403 true
"pcdivorced" 1.0 0 -955883 true

MONITOR
381
254
448
299
NIL
pcmarried
1
1
11

MONITOR
319
254
376
299
NIL
pcsingle
1
1
11

MONITOR
452
254
511
299
NIL
pccohab
1
1
11

MONITOR
453
310
523
355
NIL
pcseparated
1
1
11

MONITOR
384
309
448
354
NIL
pcwidowed
1
1
11

MONITOR
319
308
380
353
NIL
pcdivorced
1
1
11

PLOT
580
93
806
244
Household Size
Years
%
0.0
10.0
0.0
10.0
true
true
PENS
"h1" 1.0 0 -16777216 true
"h2" 1.0 0 -6459832 true
"h3" 1.0 0 -2674135 true
"h4" 1.0 0 -10899396 true

MONITOR
556
255
613
300
NIL
h1
1
1
11

MONITOR
620
255
677
300
NIL
h2
1
1
11

MONITOR
685
255
742
300
NIL
h3
1
1
11

MONITOR
750
256
807
301
NIL
h4
1
1
11

MONITOR
106
311
180
356
NIL
meanelectric
0
1
11

MONITOR
20
372
90
417
NIL
pcfood
2
1
11

MONITOR
105
373
178
418
NIL
pchousing
2
1
11

MONITOR
22
428
89
473
NIL
pcgas
2
1
11

MONITOR
105
429
178
474
NIL
pcelectric
2
1
11

PLOT
15
489
281
639
Mean Monthly Household Income
Years

0.0
10.0
0.0
10.0
true
false
PENS
"income" 1.0 1 -2674135 true

SLIDER
372
10
593
43
change_in_real_income%
change_in_real_income%
-10
10
3
1
1
NIL
HORIZONTAL

PLOT
811
93
1101
243
incomeband
Years
%
0.0
10.0
0.0
10.0
true
true
PENS
"pclow" 1.0 0 -2674135 true
"pcmedium" 1.0 0 -1184463 true
"pchigh" 1.0 0 -10899396 true

TEXTBOX
628
10
975
53
Income Change Model 3
32
15.0
1

@#$#@#$#@
WHAT IS IT?
-----------
A simulation of the effect of changes in real household income on expenditure for food, housing, gas, electricity and mortgage.


HOW IT WORKS
------------
The program loads in data from the 1991 BHPS covering a range of demographic and expenditure parameters. It then advances the population over time. Real household income is varied to determine its effect on budget shares for food, housing, gas, electricity and mortgage.

After loading in all the data, households are classified into six demographic types:

1. Single person, non-pensionable (1 person aged under 65 if male and under 60 if female)

2. Single person, pensionable (1 person aged 65 and over if male and 60 or over if female)

3. Two adults (two individuals, both aged 16 or over)

4. Two adults with one or more children (two individuals aged 16 or over as well as one or more aged under 16)

5. One adult with one or more children (one person aged 16 or over with one or more aged under 16)

6. All other types

Then, for each simulated year, household incomes are changed by the amount shown on the change_in_real_income slider. Each household is processed in turn. First, the income of the current household is changed by the appropriate number of percentage points. The new income is stored as the 'targetincome'. Next a search is made for a household that has the same demographic type and also has a current income as close as possible to the current household's targetincome. Since there may not be an exact match on incomes, by an even probability, the next higher or lower income household is chosen. If there is no higher or lower income household, the current household's income and expenditures are all varied in proportion to the desired precentage change in income.

When an appropriate household is located, its income and expenditure pattern is copied to a set of global variables. These are then copied to a set of temporary variables within the current household. The new mean household income for the whole population is then calculated and compared with what it would be when all household incomes have been changed by the required percentage. Households continue to be processed until the actual mean household income reaches this figure. At this point, all household temporary variables are copied to the household's working expenditure variables ready for the next year's processing or for output.

When all the households have been updated, the program calculates some summary statistics then outputs them on the user interface as graphs or numerically.

The program runs forward in time for the number of years specified on the stop_after input box.


HOW TO USE IT
-------------
First set how many households to model; up to about 10,000. A few hundred might be a good starting value because the more there are the longer it takes to run. However, the variation in the results is greater with a smaller initial sample.

Next, set the number of simulated years the program is to run for in the stop_after input box. Then load in the data by clicking on the 'setup' button. After this, the model can be run by clicking on the 'go' button.


THINGS TO NOTICE
----------------
The graph on the top left of the user interface shows the share of household income for food, housing, electricity, gas and mortgage. This is updated at the end of each simulated year. It shows that, as mean household income rises, the budget share taken up by food decreases. This is in agreement with Engel's law. It also shows that the budget share for housing and mortgage varies less with changes to household income. The mean household expenditure and percentage of income taken up by each item is shown in the boxes below the graph.

The population make-up and household size distribution are shown on the next two graphs. They do not change because there is no demographic updating in this model. The next graph shows the proportion of households classified as low income (below 10,000 per year), medium income (10,000 to 20,000 per year) and high income (20,000 and over per year).

Mean monthly household income is shown in the graph on the left side of the user interface below the budget share output boxes. Total population and mean household size are shown below the income bar graph.

The grid area shows households and occupants. Right clicking makes it possible to inspect the current state of the variables for households and individuals.


THINGS TO TRY
-------------
Household income can be varied using the siders to see what effect it has on spending patterns. The number of households represented in the simulation can be set to various levels at the start to see the effect on run time and variability of the results.


EXTENDING THE MODEL
-------------------
It would be possible to produce outputs for separate groups such as income bands or type of household.


CREDITS AND REFERENCES
----------------------
Developed by Tony Lawson as part of a PhD project at the University of Essex
to develop a microsimulation of household demand. Sponsored by the ESRC and BT.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
